/**
 * @export 函数节流:设置一个可执行状态,interval时间内不再调用就触发fn
 */
export function throttle (fn, interval = 300, scope) {
  let status = false;
  return function () {
    if (status) return;
    status = true;
    fn.apply(scope || this, arguments);
    setTimeout(() => {
      status = false;
    }, interval);
  };
}

/**
 * @export 函数防抖:每次调用重置计时器,delay时间内不再调用就触发fn
 * @function 页面内防抖 clearTimeout(this.timer); this.timer = null; this.timer = setTimeout(() => { if (_this.timer) _query(); }, 300);
 * @function 页面内防抖, 在methods内直接引用debounce, querySearch: debounce(async function(done) { ... })
 */
export function debounce (fn, delay = 300) {
  var timer;
  return function () {
    var args = arguments;
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
    timer = setTimeout(() => {
      fn.apply(this, args);
    }, delay);
  };
}

/**
 * @export 对象深度拷贝
 */
export function deepClone (source) {
  // 判断复制的目标是数组还是对象
  const targetObj = source.constructor === Array ? [] : {};
  for (const keys in source) {
    if (source.hasOwnProperty(keys)) {
      if (source[keys] && typeof source[keys] === "object") {
        // 如果值是对象，就递归一下
        targetObj[keys] = source[keys].constructor === Array ? [] : {};
        targetObj[keys] = deepClone(source[keys]);
      } else {
        // 如果不是，就直接赋值
        targetObj[keys] = source[keys];
      }
    }
  }
  return targetObj;
}

/**
 * @export 图片地址转base64:异步
 */
export function imageUrlToBase64Image (url) {
  return new Promise((resolve) => {
    var image = new Image();
    image.src = url + "?v=" + Math.random(); // 处理缓存
    image.crossOrigin = "*"; // 支持跨域图片
    image.onload = () => {
      var canvas = document.createElement("canvas");
      canvas.width = image.width;
      canvas.height = image.height;
      var context = canvas.getContext("2d");
      context.drawImage(image, 0, 0, image.width, image.height);
      var quality = 0.8;
      resolve(canvas.toDataURL("image/jpeg", quality));
    };
  });
}

export default {
  throttle, debounce
}
